from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.sample_material import SampleMaterial
from ...types import Response


def _get_kwargs(
    sample_material_id: int,
) -> dict[str, Any]:
    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": f"/sample_materials/{sample_material_id}",
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> HTTPValidationError | SampleMaterial | None:
    if response.status_code == 200:
        response_200 = SampleMaterial.from_dict(response.json())

        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[HTTPValidationError | SampleMaterial]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    sample_material_id: int,
    *,
    client: AuthenticatedClient | Client,
) -> Response[HTTPValidationError | SampleMaterial]:
    """Get Sample Material

    Args:
        sample_material_id (int):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | SampleMaterial]
    """

    kwargs = _get_kwargs(
        sample_material_id=sample_material_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    sample_material_id: int,
    *,
    client: AuthenticatedClient | Client,
) -> HTTPValidationError | SampleMaterial | None:
    """Get Sample Material

    Args:
        sample_material_id (int):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | SampleMaterial
    """

    return sync_detailed(
        sample_material_id=sample_material_id,
        client=client,
    ).parsed


async def asyncio_detailed(
    sample_material_id: int,
    *,
    client: AuthenticatedClient | Client,
) -> Response[HTTPValidationError | SampleMaterial]:
    """Get Sample Material

    Args:
        sample_material_id (int):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | SampleMaterial]
    """

    kwargs = _get_kwargs(
        sample_material_id=sample_material_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    sample_material_id: int,
    *,
    client: AuthenticatedClient | Client,
) -> HTTPValidationError | SampleMaterial | None:
    """Get Sample Material

    Args:
        sample_material_id (int):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | SampleMaterial
    """

    return (
        await asyncio_detailed(
            sample_material_id=sample_material_id,
            client=client,
        )
    ).parsed
